@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -12,6 +12,7 @@
 #include "katze-array.h"
 
 #include "katze-utils.h"
+#include "marshal.h"
 
 #include <glib/gi18n.h>
 #include <string.h>
@@ -26,40 +27,71 @@
 
 struct _KatzeArray
 {
-    KatzeList parent_instance;
+    KatzeItem parent_instance;
 
     GType type;
+    GList* items;
 };
 
 struct _KatzeArrayClass
 {
-    KatzeListClass parent_class;
+    KatzeItemClass parent_class;
+
+    /* Signals */
+    void
+    (*add_item)               (KatzeArray* array,
+                               gpointer    item);
+    void
+    (*remove_item)            (KatzeArray* array,
+                               gpointer    item);
+    void
+    (*move_item)              (KatzeArray* array,
+                               gpointer    item,
+                               gint        index);
+    void
+    (*clear)                  (KatzeArray* array);
 };
 
-G_DEFINE_TYPE (KatzeArray, katze_array, KATZE_TYPE_LIST)
+G_DEFINE_TYPE (KatzeArray, katze_array, KATZE_TYPE_ITEM);
+
+enum {
+    ADD_ITEM,
+    REMOVE_ITEM,
+    MOVE_ITEM,
+    CLEAR,
+
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL];
 
 static void
 katze_array_finalize (GObject* object);
 
 static void
-_katze_array_add_item (KatzeList* list,
-                       gpointer   item)
+_katze_array_add_item (KatzeArray* array,
+                       gpointer    item)
 {
-    if (katze_array_is_a ((KatzeArray*)list, G_TYPE_OBJECT))
+    if (katze_array_is_a (array, G_TYPE_OBJECT))
     {
+        GType type = G_OBJECT_TYPE (item);
+
+        g_return_if_fail (katze_array_is_a (array, type));
         g_object_ref (item);
-        if (KATZE_IS_ITEM (item))
-            katze_item_set_parent (item, list);
+        if (g_type_is_a (type, KATZE_TYPE_ITEM))
+            katze_item_set_parent (item, array);
     }
-    KATZE_LIST_CLASS (katze_array_parent_class)->add_item (list, item);
+
+    array->items = g_list_append (array->items, item);
 }
 
 static void
-_katze_array_remove_item (KatzeList* list,
+_katze_array_remove_item (KatzeArray* array,
                           gpointer   item)
 {
-    KATZE_LIST_CLASS (katze_array_parent_class)->remove_item (list, item);
-    if (katze_array_is_a ((KatzeArray*)list, G_TYPE_OBJECT))
+    array->items = g_list_remove (array->items, item);
+
+    if (katze_array_is_a (array, G_TYPE_OBJECT))
     {
         if (KATZE_IS_ITEM (item))
             katze_item_set_parent (item, NULL);
@@ -67,40 +99,127 @@ _katze_array_remove_item (KatzeList* list,
     }
 }
 
+static void
+_katze_array_move_item (KatzeArray* array,
+                        gpointer    item,
+                        gint        position)
+{
+    array->items = g_list_remove (array->items, item);
+    array->items = g_list_insert (array->items, item, position);
+}
+
+static void
+_katze_array_clear (KatzeArray* array)
+{
+    guint n;
+    guint i;
+    GObject* item;
+
+    n = g_list_length (array->items);
+    for (i = 0; i < n; i++)
+    {
+        if ((item = g_list_nth_data (array->items, i)))
+            katze_array_remove_item (array, item);
+    }
+    g_list_free (array->items);
+    array->items = NULL;
+}
+
 static void
 katze_array_class_init (KatzeArrayClass* class)
 {
     GObjectClass* gobject_class;
-    KatzeListClass* katzelist_class;
 
     gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = katze_array_finalize;
 
-    katzelist_class = KATZE_LIST_CLASS (class);
-    katzelist_class->add_item = _katze_array_add_item;
-    katzelist_class->remove_item = _katze_array_remove_item;
+    signals[ADD_ITEM] = g_signal_new (
+        "add-item",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (KatzeArrayClass, add_item),
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__POINTER,
+        G_TYPE_NONE, 1,
+        G_TYPE_POINTER);
+
+    signals[REMOVE_ITEM] = g_signal_new (
+        "remove-item",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (KatzeArrayClass, remove_item),
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__POINTER,
+        G_TYPE_NONE, 1,
+        G_TYPE_POINTER);
+
+    /**
+     * KatzeArray::move-item:
+     * @array: the object on which the signal is emitted
+     * @item: the item being moved
+     * @position: the new position of the item
+     *
+     * An item is moved to a new position.
+     *
+     * Since: 0.1.6
+     **/
+    signals[MOVE_ITEM] = g_signal_new (
+        "move-item",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (KatzeArrayClass, move_item),
+        0,
+        NULL,
+        katze_cclosure_marshal_VOID__POINTER_INT,
+        G_TYPE_NONE, 2,
+        G_TYPE_POINTER,
+        G_TYPE_INT);
+
+    signals[CLEAR] = g_signal_new (
+        "clear",
+        G_TYPE_FROM_CLASS (class),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+        G_STRUCT_OFFSET (KatzeArrayClass, clear),
+        0,
+        NULL,
+        g_cclosure_marshal_VOID__VOID,
+        G_TYPE_NONE, 0);
+
+    gobject_class = G_OBJECT_CLASS (class);
+    gobject_class->finalize = katze_array_finalize;
+
+    class->add_item = _katze_array_add_item;
+    class->remove_item = _katze_array_remove_item;
+    class->move_item = _katze_array_move_item;
+    class->clear = _katze_array_clear;
 }
 
 static void
 katze_array_init (KatzeArray* array)
 {
     array->type = G_TYPE_NONE;
+    array->items = NULL;
 }
 
 static void
 katze_array_finalize (GObject* object)
 {
     KatzeArray* array;
-    guint n, i;
+    guint i;
 
     array = KATZE_ARRAY (object);
     if (katze_array_is_a (array, G_TYPE_OBJECT))
     {
-        n = katze_list_get_length ((KatzeList*)array);
-        for (i = 0; i < n; i++)
-            g_object_unref (katze_list_get_nth_item ((KatzeList*)array, i));
+        gpointer item;
+        i = 0;
+        while ((item = g_list_nth_data (array->items, i++)))
+            g_object_unref (item);
     }
 
+    g_list_free (array->items);
+
     G_OBJECT_CLASS (katze_array_parent_class)->finalize (object);
 }
 
@@ -111,15 +230,10 @@ katze_array_finalize (GObject* object)
  * Creates a new #KatzeArray for @type items.
  *
  * You may only add items of the given type or inherited
- * from it to this array.
- *
+ * from it to this array *if* @type is an #GObject type.
  * The array will keep a reference on each object until
  * it is removed from the array.
  *
- * Note: While you *can* (currently) use #KatzeList accessors
- * to circumvent type safety, you are *encouraged* to use
- * only #KatzeArray accessors, or behaviour is undefined.
- *
  * Return value: a new #KatzeArray
  **/
 KatzeArray*
@@ -154,7 +268,7 @@ katze_array_is_a (KatzeArray* array,
 /**
  * katze_array_add_item:
  * @array: a #KatzeArray
- * @item: a #GObject
+ * @item: an item
  *
  * Adds an item to the array.
  *
@@ -165,16 +279,14 @@ katze_array_add_item (KatzeArray* array,
                       gpointer    item)
 {
     g_return_if_fail (KATZE_IS_ARRAY (array));
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-        g_return_if_fail (katze_array_is_a (array, G_OBJECT_TYPE (item)));
 
-    g_signal_emit_by_name (array, "add-item", item);
+    g_signal_emit (array, signals[ADD_ITEM], 0, item);
 }
 
 /**
  * katze_array_remove_item:
  * @array: a #KatzeArray
- * @item: a #GObject
+ * @item: an item
  *
  * Removes an item from the array.
  *
@@ -185,10 +297,8 @@ katze_array_remove_item (KatzeArray* array,
                          gpointer    item)
 {
     g_return_if_fail (KATZE_IS_ARRAY (array));
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-        g_return_if_fail (katze_array_is_a (array, G_OBJECT_TYPE (item)));
 
-    g_signal_emit_by_name (array, "remove-item", item);
+    g_signal_emit (array, signals[REMOVE_ITEM], 0, item);
 }
 
 /**
@@ -206,14 +316,14 @@ katze_array_get_nth_item (KatzeArray* array,
 {
     g_return_val_if_fail (KATZE_IS_ARRAY (array), NULL);
 
-    return katze_list_get_nth_item (KATZE_LIST (array), n);
+    return g_list_nth_data (array->items, n);
 }
 
 /**
  * katze_array_is_empty:
  * @array: a #KatzeArray
  *
- * Determines if @array is empty.
+ * Determines whether @array is empty.
  *
  * Return value: an item, or %NULL
  **/
@@ -222,7 +332,7 @@ katze_array_is_empty (KatzeArray* array)
 {
     g_return_val_if_fail (KATZE_IS_ARRAY (array), TRUE);
 
-    return katze_list_is_empty (KATZE_LIST (array));
+    return !g_list_nth_data (array->items, 0);
 }
 
 /**
@@ -239,10 +349,8 @@ katze_array_get_item_index (KatzeArray* array,
                             gpointer    item)
 {
     g_return_val_if_fail (KATZE_IS_ARRAY (array), -1);
-    if (katze_array_is_a (array, G_TYPE_OBJECT))
-        g_return_val_if_fail (katze_array_is_a (array, G_OBJECT_TYPE (item)), -1);
 
-    return katze_list_get_item_index (KATZE_LIST (array), item);
+    return g_list_index (array->items, item);
 }
 
 /**
@@ -264,23 +372,17 @@ gpointer
 katze_array_find_token (KatzeArray*  array,
                         const gchar* token)
 {
-    guint n, i;
+    guint i;
     gpointer item;
-    const gchar* found_token;
-
-    g_return_val_if_fail (KATZE_IS_ARRAY (array), NULL);
 
     if (!katze_array_is_a (array, G_TYPE_OBJECT))
         return NULL;
 
-    n = katze_list_get_length ((KatzeList*)array);
-    for (i = 0; i < n; i++)
+    i = 0;
+    while ((item = g_list_nth_data (array->items, i++)))
     {
-        item = katze_list_get_nth_item ((KatzeList*)array, i);
-        if (!g_type_is_a (G_OBJECT_TYPE (item), KATZE_TYPE_ITEM))
-            continue;
-        found_token = katze_item_get_token ((KatzeItem*)item);
-        if (found_token && !strcmp (found_token, token))
+        const gchar* found_token = katze_item_get_token ((KatzeItem*)item);
+        if (!g_strcmp0 (found_token, token))
             return item;
     }
     return NULL;
@@ -299,7 +401,27 @@ katze_array_get_length (KatzeArray* array)
 {
     g_return_val_if_fail (KATZE_IS_ARRAY (array), 0);
 
-    return katze_list_get_length (KATZE_LIST (array));
+    return g_list_length (array->items);
+}
+
+/**
+ * katze_array_move_item:
+ * @array: a #KatzeArray
+ * @item: the item being moved
+ * @position: the new position of the item
+ *
+ * Moves @item to the position @position.
+ *
+ * Since: 0.1.6
+ **/
+void
+katze_array_move_item (KatzeArray* array,
+                       gpointer    item,
+                       gint        position)
+{
+    g_return_if_fail (KATZE_IS_ARRAY (array));
+
+    g_signal_emit (array, signals[MOVE_ITEM], 0, item, position);
 }
 
 /**
@@ -313,5 +435,5 @@ katze_array_clear (KatzeArray* array)
 {
     g_return_if_fail (KATZE_IS_ARRAY (array));
 
-    katze_list_clear (KATZE_LIST (array));
+    g_signal_emit (array, signals[CLEAR], 0);
 }
