@@ -150,6 +150,9 @@ proxy_object_notify_string_cb (GObject*    object,
  *         choosing an existing filename, encoded as an URI.
  *     "font": the widget created will be particularly suitable for
  *         choosing a font from installed fonts.
+ *     Since 0.1.6 the following hints are also supported:
+ *     "toggle": the widget created will be an empty toggle button. This
+ *         is only supported with boolean properties.
  *
  * Any other values for @hint are silently ignored.
  *
@@ -193,7 +196,10 @@ katze_property_proxy (gpointer     object,
         gchar* notify_property;
         gboolean toggled = katze_object_get_boolean (object, property);
 
-        widget = gtk_check_button_new_with_label (gettext (nick));
+        if (_hint == g_intern_string ("toggle"))
+            widget = gtk_toggle_button_new ();
+        else
+            widget = gtk_check_button_new_with_label (gettext (nick));
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), toggled);
         g_signal_connect (widget, "toggled",
                           G_CALLBACK (proxy_toggle_button_toggled_cb), object);
