@@ -14,20 +14,15 @@
     #include <config.h>
 #endif
 
+#include "midori.h"
 #include "midori-addons.h"
-#include "midori-app.h"
+#include "midori-array.h"
 #include "midori-bookmarks.h"
-#include "midori-browser.h"
 #include "midori-console.h"
-#include "midori-extension.h"
 #include "midori-extensions.h"
 #include "midori-history.h"
-#include "midori-panel.h"
-#include "midori-preferences.h"
 #include "midori-plugins.h"
 #include "midori-transfers.h"
-#include "midori-view.h"
-#include "midori-websettings.h"
 
 #include "sokoke.h"
 #include "compat.h"
@@ -40,16 +35,11 @@
 #else
     #define is_writable(_cfg_filename) 1
 #endif
+#include <stdlib.h>
 #include <string.h>
-#include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <webkit/webkit.h>
 
-#if HAVE_LIBXML
-    #include <libxml/parser.h>
-    #include <libxml/tree.h>
-#endif
-
 #if HAVE_SQLITE
     #include <sqlite3.h>
 #endif
@@ -59,6 +49,10 @@
     #include <locale.h>
 #endif
 
+#if HAVE_HILDON
+    #include <libosso.h>
+#endif
+
 #define MIDORI_HISTORY_ERROR g_quark_from_string("MIDORI_HISTORY_ERROR")
 
 typedef enum
@@ -71,10 +65,7 @@ typedef enum
 static gchar*
 build_config_filename (const gchar* filename)
 {
-    static gchar* path = NULL;
-
-    if (!path)
-        path = g_build_filename (g_get_user_config_dir (), PACKAGE_NAME, NULL);
+    const gchar* path = sokoke_set_config_dir (NULL);
     g_mkdir_with_parents (path, 0700);
     return g_build_filename (path, filename, NULL);
 }
@@ -164,6 +155,7 @@ settings_new_from_file (const gchar* filename)
         else
             g_warning (_("Invalid configuration value '%s'"), property);
     }
+    g_free (pspecs);
     return settings;
 }
 
@@ -198,10 +190,11 @@ settings_save_to_file (MidoriWebSettings* settings,
         }
         if (type == G_TYPE_PARAM_STRING)
         {
-            const gchar* string;
+            gchar* string;
             g_object_get (settings, property, &string, NULL);
             g_key_file_set_string (key_file, "settings", property,
                                    string ? string : "");
+            g_free (string);
         }
         else if (type == G_TYPE_PARAM_INT)
         {
@@ -234,6 +227,7 @@ settings_save_to_file (MidoriWebSettings* settings,
         else
             g_warning (_("Invalid configuration value '%s'"), property);
     }
+    g_free (pspecs);
     saved = sokoke_key_file_save_to_file (key_file, filename, error);
     g_key_file_free (key_file);
     return saved;
@@ -276,6 +270,7 @@ search_engines_new_from_file (const gchar* filename,
         }
         katze_array_add_item (search_engines, item);
     }
+    g_free (pspecs);
     g_strfreev (engines);
     g_key_file_free (key_file);
     return search_engines;
@@ -287,7 +282,7 @@ search_engines_save_to_file (KatzeArray*  search_engines,
                              GError**     error)
 {
     GKeyFile* key_file;
-    guint n, i, j, n_properties;
+    guint i, j, n_properties;
     KatzeItem* item;
     const gchar* name;
     GParamSpec** pspecs;
@@ -296,12 +291,11 @@ search_engines_save_to_file (KatzeArray*  search_engines,
     gboolean saved;
 
     key_file = g_key_file_new ();
-    n = katze_array_get_length (search_engines);
     pspecs = g_object_class_list_properties (G_OBJECT_GET_CLASS (search_engines),
                                              &n_properties);
-    for (i = 0; i < n; i++)
+    i = 0;
+    while ((item = katze_array_get_nth_item (search_engines, i++)))
     {
-        item = katze_array_get_nth_item (search_engines, i);
         name = katze_item_get_name (item);
         for (j = 0; j < n_properties; j++)
         {
@@ -314,191 +308,13 @@ search_engines_save_to_file (KatzeArray*  search_engines,
             g_free (value);
         }
     }
+    g_free (pspecs);
     saved = sokoke_key_file_save_to_file (key_file, filename, error);
     g_key_file_free (key_file);
 
     return saved;
 }
 
-#if HAVE_LIBXML
-static KatzeItem*
-katze_item_from_xmlNodePtr (xmlNodePtr cur)
-{
-    KatzeItem* item;
-    xmlChar* key;
-
-    item = katze_item_new ();
-    key = xmlGetProp (cur, (xmlChar*)"href");
-    katze_item_set_uri (item, (gchar*)key);
-    g_free (key);
-
-    cur = cur->xmlChildrenNode;
-    while (cur)
-    {
-        if (!xmlStrcmp (cur->name, (const xmlChar*)"title"))
-        {
-            key = xmlNodeGetContent (cur);
-            katze_item_set_name (item, g_strstrip ((gchar*)key));
-            g_free (key);
-        }
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"desc"))
-        {
-            key = xmlNodeGetContent (cur);
-            katze_item_set_text (item, g_strstrip ((gchar*)key));
-            g_free (key);
-        }
-        cur = cur->next;
-    }
-    return item;
-}
-
-/* Create an array from an xmlNodePtr */
-static KatzeArray*
-katze_array_from_xmlNodePtr (xmlNodePtr cur)
-{
-    KatzeArray* array;
-    xmlChar* key;
-    KatzeItem* item;
-
-    array = katze_array_new (KATZE_TYPE_ARRAY);
-
-    key = xmlGetProp (cur, (xmlChar*)"folded");
-    if (key)
-    {
-        /* if (!g_ascii_strncasecmp ((gchar*)key, "yes", 3))
-            folder->folded = TRUE;
-        else if (!g_ascii_strncasecmp ((gchar*)key, "no", 2))
-            folder->folded = FALSE;
-        else
-            g_warning ("XBEL: Unknown value for folded."); */
-        xmlFree (key);
-    }
-
-    cur = cur->xmlChildrenNode;
-    while (cur)
-    {
-        if (!xmlStrcmp (cur->name, (const xmlChar*)"title"))
-        {
-            key = xmlNodeGetContent (cur);
-            katze_item_set_name (KATZE_ITEM (array), g_strstrip ((gchar*)key));
-        }
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"desc"))
-        {
-            key = xmlNodeGetContent (cur);
-            katze_item_set_text (KATZE_ITEM (array), g_strstrip ((gchar*)key));
-        }
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"folder"))
-        {
-            item = (KatzeItem*)katze_array_from_xmlNodePtr (cur);
-            katze_array_add_item (array, item);
-        }
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"bookmark"))
-        {
-            item = katze_item_from_xmlNodePtr (cur);
-            katze_array_add_item (array, item);
-        }
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"separator"))
-        {
-            item = katze_item_new ();
-            katze_array_add_item (array, item);
-        }
-        cur = cur->next;
-    }
-    return array;
-}
-
-/* Loads the contents from an xmlNodePtr into an array. */
-static gboolean
-katze_array_from_xmlDocPtr (KatzeArray* array,
-                            xmlDocPtr   doc)
-{
-    xmlNodePtr cur;
-    xmlChar* version;
-    gchar* value;
-    KatzeItem* item;
-
-    cur = xmlDocGetRootElement (doc);
-    version = xmlGetProp (cur, (xmlChar*)"version");
-    if (xmlStrcmp (version, (xmlChar*)"1.0"))
-        g_warning ("XBEL version is not 1.0.");
-    xmlFree (version);
-
-    value = (gchar*)xmlGetProp (cur, (xmlChar*)"title");
-    katze_item_set_name (KATZE_ITEM (array), value);
-    g_free (value);
-
-    value = (gchar*)xmlGetProp (cur, (xmlChar*)"desc");
-    katze_item_set_text (KATZE_ITEM (array), value);
-    g_free (value);
-
-    if ((cur = xmlDocGetRootElement (doc)) == NULL)
-    {
-        /* Empty document */
-        return FALSE;
-    }
-    if (xmlStrcmp (cur->name, (const xmlChar*)"xbel"))
-    {
-        /* Wrong document kind */
-        return FALSE;
-    }
-    cur = cur->xmlChildrenNode;
-    while (cur)
-    {
-        item = NULL;
-        if (!xmlStrcmp (cur->name, (const xmlChar*)"folder"))
-            item = (KatzeItem*)katze_array_from_xmlNodePtr (cur);
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"bookmark"))
-            item = katze_item_from_xmlNodePtr (cur);
-        else if (!xmlStrcmp (cur->name, (const xmlChar*)"separator"))
-            item = katze_item_new ();
-        /*else if (!xmlStrcmp (cur->name, (const xmlChar*)"info"))
-            item = katze_xbel_parse_info (xbel, cur);*/
-        if (item)
-            katze_array_add_item (array, item);
-        cur = cur->next;
-    }
-    return TRUE;
-}
-
-static gboolean
-katze_array_from_file (KatzeArray*  array,
-                       const gchar* filename,
-                       GError**     error)
-{
-    xmlDocPtr doc;
-
-    g_return_val_if_fail (katze_array_is_a (array, KATZE_TYPE_ITEM), FALSE);
-    g_return_val_if_fail (filename != NULL, FALSE);
-
-    if (!g_file_test (filename, G_FILE_TEST_EXISTS))
-    {
-        /* File doesn't exist */
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_NOENT,
-                                      _("File not found."));
-        return FALSE;
-    }
-
-    if ((doc = xmlParseFile (filename)) == NULL)
-    {
-        /* No valid xml or broken encoding */
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                                      _("Malformed document."));
-        return FALSE;
-    }
-
-    if (!katze_array_from_xmlDocPtr (array, doc))
-    {
-        /* Parsing failed */
-        xmlFreeDoc (doc);
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_FAILED,
-                                      _("Malformed document."));
-        return FALSE;
-    }
-    xmlFreeDoc (doc);
-    return TRUE;
-}
-#endif
-
 #if HAVE_SQLITE
 /* Open database 'dbname' */
 static sqlite3*
@@ -800,7 +616,7 @@ midori_history_initialize (KatzeArray*  array,
 {
     sqlite3* db;
     KatzeItem* item;
-    gint i, n;
+    gint i;
     gboolean has_day;
 
     has_day = FALSE;
@@ -846,10 +662,9 @@ midori_history_initialize (KatzeArray*  array,
                            error))
         return NULL;
 
-    n = katze_array_get_length (array);
-    for (i = 0; i < n; i++)
+    i = 0;
+    while ((item = katze_array_get_nth_item (array, i++)))
     {
-        item = katze_array_get_nth_item (array, i);
         g_signal_connect_after (item, "add-item",
             G_CALLBACK (midori_history_add_item_cb), db);
         g_signal_connect (item, "remove-item",
@@ -940,138 +755,6 @@ midori_search_engines_modify_cb (KatzeArray* array,
     g_free (config_file);
 }
 
-static gchar*
-_simple_xml_element (const gchar* name,
-                     const gchar* value)
-{
-    gchar* value_escaped;
-    gchar* markup;
-
-    if (!value)
-        return g_strdup ("");
-    value_escaped = g_markup_escape_text (value, -1);
-    markup = g_strdup_printf ("<%s>%s</%s>\n", name, value_escaped, name);
-    g_free (value_escaped);
-    return markup;
-}
-
-static gchar*
-katze_item_to_data (KatzeItem* item)
-{
-    gchar* markup;
-
-    g_return_val_if_fail (KATZE_IS_ITEM (item), NULL);
-
-    markup = NULL;
-    if (KATZE_IS_ARRAY (item))
-    {
-        GString* _markup = g_string_new (NULL);
-        guint n = katze_array_get_length (KATZE_ARRAY (item));
-        guint i;
-        for (i = 0; i < n; i++)
-        {
-            KatzeItem* _item = katze_array_get_nth_item (KATZE_ARRAY (item), i);
-            gchar* item_markup = katze_item_to_data (_item);
-            g_string_append (_markup, item_markup);
-            g_free (item_markup);
-        }
-        /* gchar* folded = item->folded ? NULL : g_strdup_printf (" folded=\"no\""); */
-        gchar* title = _simple_xml_element ("title", katze_item_get_name (item));
-        gchar* desc = _simple_xml_element ("desc", katze_item_get_text (item));
-        markup = g_strdup_printf ("<folder%s>\n%s%s%s</folder>\n",
-                                  "" /* folded ? folded : "" */,
-                                  title, desc,
-                                  g_string_free (_markup, FALSE));
-        /* g_free (folded); */
-        g_free (title);
-        g_free (desc);
-    }
-    else if (katze_item_get_uri (item))
-    {
-        gchar* href_escaped = g_markup_escape_text (katze_item_get_uri (item), -1);
-        gchar* href = g_strdup_printf (" href=\"%s\"", href_escaped);
-        g_free (href_escaped);
-        gchar* title = _simple_xml_element ("title", katze_item_get_name (item));
-        gchar* desc = _simple_xml_element ("desc", katze_item_get_text (item));
-        markup = g_strdup_printf ("<bookmark%s>\n%s%s%s</bookmark>\n",
-                                  href,
-                                  title, desc,
-                                  "");
-        g_free (href);
-        g_free (title);
-        g_free (desc);
-    }
-    else
-        markup = g_strdup ("<separator/>\n");
-    return markup;
-}
-
-static gchar*
-katze_array_to_xml (KatzeArray* array,
-                    GError**    error)
-{
-    GString* inner_markup;
-    guint i, n;
-    KatzeItem* item;
-    gchar* item_xml;
-    gchar* title;
-    gchar* desc;
-    gchar* outer_markup;
-
-    g_return_val_if_fail (katze_array_is_a (array, KATZE_TYPE_ITEM), NULL);
-
-    inner_markup = g_string_new (NULL);
-    n = katze_array_get_length (array);
-    for (i = 0; i < n; i++)
-    {
-        item = katze_array_get_nth_item (array, i);
-        item_xml = katze_item_to_data (item);
-        g_string_append (inner_markup, item_xml);
-        g_free (item_xml);
-    }
-
-    title = _simple_xml_element ("title", katze_item_get_name (KATZE_ITEM (array)));
-    desc = _simple_xml_element ("desc", katze_item_get_text (KATZE_ITEM (array)));
-    outer_markup = g_strdup_printf (
-                   "%s%s<xbel version=\"1.0\">\n%s%s%s</xbel>\n",
-                   "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n",
-                   "<!DOCTYPE xbel PUBLIC \"+//IDN python.org//DTD "
-                   "XML Bookmark Exchange Language 1.0//EN//XML\" "
-                   "\"http://www.python.org/topics/xml/dtds/xbel-1.0.dtd\">\n",
-                   title,
-                   desc,
-                   g_string_free (inner_markup, FALSE));
-    g_free (title);
-    g_free (desc);
-
-    return outer_markup;
-}
-
-static gboolean
-katze_array_to_file (KatzeArray*  array,
-                     const gchar* filename,
-                     GError**     error)
-{
-    gchar* data;
-    FILE* fp;
-
-    g_return_val_if_fail (katze_array_is_a (array, KATZE_TYPE_ITEM), FALSE);
-    g_return_val_if_fail (filename, FALSE);
-
-    if (!(data = katze_array_to_xml (array, error)))
-        return FALSE;
-    if (!(fp = fopen (filename, "w")))
-    {
-        *error = g_error_new_literal (G_FILE_ERROR, G_FILE_ERROR_ACCES,
-                                      _("Writing failed."));
-        return FALSE;
-    }
-    fputs (data, fp);
-    fclose (fp);
-    g_free (data);
-    return TRUE;
-}
-
 static void
 midori_bookmarks_notify_item_cb (KatzeArray* folder,
                                  GParamSpec* pspec,
@@ -1082,7 +765,7 @@ midori_bookmarks_notify_item_cb (KatzeArray* folder,
 
     config_file = build_config_filename ("bookmarks.xbel");
     error = NULL;
-    if (!katze_array_to_file (bookmarks, config_file, &error))
+    if (!midori_array_to_file (bookmarks, config_file, "xbel", &error))
     {
         g_warning (_("The bookmarks couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -1096,8 +779,9 @@ midori_bookmarks_add_item_cb (KatzeArray* folder,
                               KatzeArray* bookmarks);
 
 static void
-midori_bookmarks_remove_item_cb (KatzeArray* bookmarks,
-                                 GObject*    item);
+midori_bookmarks_remove_item_cb (KatzeArray* folder,
+                                 GObject*    item,
+                                 KatzeArray* bookmarks);
 
 static void
 midori_bookmarks_add_item_cb (KatzeArray* folder,
@@ -1109,7 +793,7 @@ midori_bookmarks_add_item_cb (KatzeArray* folder,
 
     config_file = build_config_filename ("bookmarks.xbel");
     error = NULL;
-    if (!katze_array_to_file (bookmarks, config_file, &error))
+    if (!midori_array_to_file (bookmarks, config_file, "xbel", &error))
     {
         g_warning (_("The bookmarks couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -1121,7 +805,7 @@ midori_bookmarks_add_item_cb (KatzeArray* folder,
         g_signal_connect_after (item, "add-item",
             G_CALLBACK (midori_bookmarks_add_item_cb), bookmarks);
         g_signal_connect_after (item, "remove-item",
-            G_CALLBACK (midori_bookmarks_remove_item_cb), NULL);
+            G_CALLBACK (midori_bookmarks_remove_item_cb), bookmarks);
     }
 
     g_signal_connect_after (item, "notify",
@@ -1129,15 +813,16 @@ midori_bookmarks_add_item_cb (KatzeArray* folder,
 }
 
 static void
-midori_bookmarks_remove_item_cb (KatzeArray* bookmarks,
-                                 GObject*    item)
+midori_bookmarks_remove_item_cb (KatzeArray* folder,
+                                 GObject*    item,
+                                 KatzeArray* bookmarks)
 {
     gchar* config_file;
     GError* error;
 
     config_file = build_config_filename ("bookmarks.xbel");
     error = NULL;
-    if (!katze_array_to_file (bookmarks, config_file, &error))
+    if (!midori_array_to_file (bookmarks, config_file, "xbel", &error))
     {
         g_warning (_("The bookmarks couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -1155,12 +840,11 @@ midori_trash_add_item_cb (KatzeArray* trash,
 {
     gchar* config_file;
     GError* error;
-    guint n;
     GObject* obsolete_item;
 
     config_file = build_config_filename ("tabtrash.xbel");
     error = NULL;
-    if (!katze_array_to_file (trash, config_file, &error))
+    if (!midori_array_to_file (trash, config_file, "xbel", &error))
     {
         /* i18n: Trash, or wastebin, containing closed tabs */
         g_warning (_("The trash couldn't be saved. %s"), error->message);
@@ -1168,8 +852,7 @@ midori_trash_add_item_cb (KatzeArray* trash,
     }
     g_free (config_file);
 
-    n = katze_array_get_length (trash);
-    if (n > 10)
+    if (katze_array_get_nth_item (trash, 10))
     {
         obsolete_item = katze_array_get_nth_item (trash, 0);
         katze_array_remove_item (trash, obsolete_item);
@@ -1185,7 +868,7 @@ midori_trash_remove_item_cb (KatzeArray* trash,
 
     config_file = build_config_filename ("tabtrash.xbel");
     error = NULL;
-    if (!katze_array_to_file (trash, config_file, &error))
+    if (!midori_array_to_file (trash, config_file, "xbel", &error))
     {
         g_warning (_("The trash couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -1244,6 +927,8 @@ midori_app_add_browser_cb (MidoriApp*     app,
     addon = g_object_new (MIDORI_TYPE_EXTENSIONS, "app", app, NULL);
     gtk_widget_show (addon);
     midori_panel_append_page (MIDORI_PANEL (panel), MIDORI_VIEWABLE (addon));
+
+    g_object_unref (panel);
 }
 
 static void
@@ -1256,7 +941,7 @@ midori_browser_session_cb (MidoriBrowser* browser,
 
     config_file = build_config_filename ("session.xbel");
     error = NULL;
-    if (!katze_array_to_file (session, config_file, &error))
+    if (!midori_array_to_file (session, config_file, "xbel", &error))
     {
         g_warning (_("The session couldn't be saved. %s"), error->message);
         g_error_free (error);
@@ -1286,8 +971,17 @@ soup_session_settings_notify_http_proxy_cb (MidoriWebSettings* settings,
         http_proxy = g_strdup (g_getenv ("http_proxy"));
     else
         http_proxy = katze_object_get_string (settings, "http-proxy");
-    /* soup_uri_new expects a non-NULL string */
-    proxy_uri = soup_uri_new (http_proxy ? http_proxy : "");
+    /* soup_uri_new expects a non-NULL string with a protocol */
+    if (http_proxy && g_str_has_prefix (http_proxy, "http://"))
+        proxy_uri = soup_uri_new (http_proxy);
+    else if (http_proxy && *http_proxy)
+    {
+        gchar* fixed_http_proxy = g_strconcat ("http://", http_proxy, NULL);
+        proxy_uri = soup_uri_new (fixed_http_proxy);
+        g_free (fixed_http_proxy);
+    }
+    else
+        proxy_uri = NULL;
     g_free (http_proxy);
     g_object_set (session, "proxy-uri", proxy_uri, NULL);
     if (proxy_uri)
@@ -1320,6 +1014,7 @@ midori_soup_session_debug (SoupSession* session)
 
 static void
 midori_soup_session_prepare (SoupSession*       session,
+                             SoupCookieJar*     cookie_jar,
                              MidoriWebSettings* settings)
 {
     SoupSessionFeature* feature;
@@ -1341,6 +1036,7 @@ midori_soup_session_prepare (SoupSession*       session,
     config_file = build_config_filename ("cookies.txt");
     g_object_set_data_full (G_OBJECT (feature), "filename",
                             config_file, (GDestroyNotify)g_free);
+    soup_session_add_feature (session, SOUP_SESSION_FEATURE (cookie_jar));
     soup_session_add_feature (session, feature);
 }
 
@@ -1375,7 +1071,7 @@ midori_create_diagnostic_dialog (MidoriWebSettings* settings,
     dialog = gtk_message_dialog_new (
         NULL, 0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK,
         _("Midori seems to have crashed the last time it was opened. "
-          "If this happend repeatedly, try one of the following options "
+          "If this happened repeatedly, try one of the following options "
           "to solve the problem."));
     gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
     gtk_window_set_title (GTK_WINDOW (dialog), g_get_application_name ());
@@ -1431,6 +1127,22 @@ midori_create_diagnostic_dialog (MidoriWebSettings* settings,
     return dialog;
 }
 
+static gboolean
+midori_load_cookie_jar (gpointer data)
+{
+    MidoriWebSettings* settings = MIDORI_WEB_SETTINGS (data);
+    SoupSession* webkit_session;
+    SoupCookieJar* jar;
+
+    webkit_session = webkit_get_default_session ();
+    jar = soup_cookie_jar_new ();
+    g_object_set_data (G_OBJECT (jar), "midori-settings", settings);
+    midori_soup_session_prepare (webkit_session, jar, settings);
+    g_object_unref (jar);
+
+    return FALSE;
+}
+
 static gboolean
 midori_load_extensions (gpointer data)
 {
@@ -1438,7 +1150,7 @@ midori_load_extensions (gpointer data)
     KatzeArray* extensions;
     const gchar* filename;
     MidoriExtension* extension;
-    guint n, i;
+    guint i;
 
     /* Load extensions */
     extensions = katze_array_new (MIDORI_TYPE_EXTENSION);
@@ -1494,12 +1206,9 @@ midori_load_extensions (gpointer data)
 
     g_object_set (app, "extensions", extensions, NULL);
 
-    n = katze_array_get_length (extensions);
-    for (i = 0; i < n; i++)
-    {
-        extension = katze_array_get_nth_item (extensions, i);
+    i = 0;
+    while ((extension = katze_array_get_nth_item (extensions, i++)))
         g_signal_emit_by_name (extension, "activate", app);
-    }
 
     return FALSE;
 }
@@ -1513,7 +1222,7 @@ midori_load_session (gpointer data)
     gchar* config_file;
     KatzeArray* session;
     KatzeItem* item;
-    guint n, i;
+    guint i;
 
     browser = midori_app_create_browser (app);
     midori_app_add_browser (app, browser);
@@ -1540,16 +1249,15 @@ midori_load_session (gpointer data)
             katze_item_set_uri (item, homepage);
             g_free (homepage);
         }
+        g_object_unref (settings);
         katze_array_add_item (_session, item);
+        g_object_unref (item);
     }
 
     session = midori_browser_get_proxy_array (browser);
-    n = katze_array_get_length (_session);
-    for (i = 0; i < n; i++)
-    {
-        item = katze_array_get_nth_item (_session, i);
+    i = 0;
+    while ((item = katze_array_get_nth_item (_session, i++)))
         midori_browser_add_item (browser, item);
-    }
     /* FIXME: Switch to the last active page */
     item = katze_array_get_nth_item (_session, 0);
     if (!strcmp (katze_item_get_uri (item), ""))
@@ -1614,6 +1322,8 @@ int
 main (int    argc,
       char** argv)
 {
+    gchar* webapp;
+    gchar* config;
     gboolean run;
     gboolean version;
     gchar** uris;
@@ -1622,6 +1332,10 @@ main (int    argc,
     GError* error;
     GOptionEntry entries[] =
     {
+       { "app", 'a', 0, G_OPTION_ARG_STRING, &webapp,
+       N_("Run ADDRESS as a web application"), N_("ADDRESS") },
+       { "config", 'c', 0, G_OPTION_ARG_FILENAME, &config,
+       N_("Use FOLDER as configuration folder"), N_("FOLDER") },
        { "run", 'r', 0, G_OPTION_ARG_NONE, &run,
        N_("Run the specified filename as javascript"), NULL },
        { "version", 'V', 0, G_OPTION_ARG_NONE, &version,
@@ -1643,13 +1357,13 @@ main (int    argc,
     gchar* uri;
     KatzeItem* item;
     gchar* uri_ready;
-    KatzeNet* net;
-    SoupSession* s_session;
-    SoupCookieJar* jar;
     #if HAVE_SQLITE
     sqlite3* db;
     gint max_history_age;
     #endif
+    #if HAVE_HILDON
+    osso_context_t* osso_context;
+    #endif
 
     #if ENABLE_NLS
     setlocale (LC_ALL, "");
@@ -1662,6 +1376,8 @@ main (int    argc,
     #endif
 
     /* Parse cli options */
+    webapp = NULL;
+    config = NULL;
     run = FALSE;
     version = FALSE;
     uris = NULL;
@@ -1674,8 +1390,7 @@ main (int    argc,
         return 1;
     }
 
-    /* libSoup uses threads, therefore if WebKit is built with libSoup
-       or Midori is using it, we need to initialize threads. */
+    /* libSoup uses threads, so we need to initialize threads. */
     if (!g_thread_supported ()) g_thread_init (NULL);
     sokoke_register_stock_items ();
     g_set_application_name (_("Midori"));
@@ -1697,11 +1412,63 @@ main (int    argc,
         return 0;
     }
 
+    /* Web Application support */
+    if (webapp)
+    {
+        MidoriBrowser* browser = midori_browser_new ();
+        settings = katze_object_get_object (browser, "settings");
+        g_object_set (settings,
+                      "show-menubar", FALSE,
+                      "show-navigationbar", TRUE,
+                      "toolbar-items", "Back,Forward,ReloadStop,Location",
+                      "show-statusbar", TRUE,
+                      NULL);
+        g_object_unref (settings);
+        g_object_set (browser, "settings", settings, NULL);
+        midori_browser_add_uri (browser, webapp);
+        g_object_set_data (G_OBJECT (browser), "locked", (void*)1);
+        g_signal_connect (browser, "destroy",
+            G_CALLBACK (gtk_main_quit), NULL);
+        gtk_widget_show (GTK_WIDGET (browser));
+        midori_browser_activate_action (browser, "Location");
+        gtk_main ();
+        return 0;
+    }
+
     /* Standalone javascript support */
     if (run)
         return midori_run_script (uris ? *uris : NULL);
 
-    app = midori_app_new ();
+    #if HAVE_HILDON
+    osso_context = osso_initialize (PACKAGE_NAME, PACKAGE_VERSION, FALSE, NULL);
+
+    if (!osso_context)
+    {
+        g_critical ("Error initializing OSSO D-Bus context - Midori");
+        return 1;
+    }
+    #endif
+
+    if (config && !g_path_is_absolute (config))
+    {
+        g_critical (_("The specified configuration folder is invalid."));
+        return 1;
+    }
+    sokoke_set_config_dir (config);
+    if (config)
+    {
+        gchar* name_hash;
+        gchar* app_name;
+        name_hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, config, -1);
+        app_name = g_strconcat ("midori", "_", name_hash, NULL);
+        g_free (name_hash);
+        app = g_object_new (MIDORI_TYPE_APP, "name", app_name, NULL);
+        g_free (app_name);
+    }
+    else
+        app = midori_app_new ();
+    g_free (config);
+
     /* FIXME: The app might be 'running' but actually showing a dialog
               after a crash, so running a new window isn't a good idea. */
     if (midori_app_instance_is_running (app))
@@ -1777,7 +1544,7 @@ main (int    argc,
     #if HAVE_LIBXML
     katze_assign (config_file, build_config_filename ("bookmarks.xbel"));
     error = NULL;
-    if (!katze_array_from_file (bookmarks, config_file, &error))
+    if (!midori_array_from_file (bookmarks, config_file, "xbel", &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
             g_string_append_printf (error_messages,
@@ -1792,7 +1559,7 @@ main (int    argc,
     {
         katze_assign (config_file, build_config_filename ("session.xbel"));
         error = NULL;
-        if (!katze_array_from_file (_session, config_file, &error))
+        if (!midori_array_from_file (_session, config_file, "xbel", &error))
         {
             if (error->code != G_FILE_ERROR_NOENT)
                 g_string_append_printf (error_messages,
@@ -1805,7 +1572,7 @@ main (int    argc,
     #if HAVE_LIBXML
     katze_assign (config_file, build_config_filename ("tabtrash.xbel"));
     error = NULL;
-    if (!katze_array_from_file (trash, config_file, &error))
+    if (!midori_array_from_file (trash, config_file, "xbel", &error))
     {
         if (error->code != G_FILE_ERROR_NOENT)
             g_string_append_printf (error_messages,
@@ -1869,24 +1636,6 @@ main (int    argc,
     }
     g_string_free (error_messages, TRUE);
 
-    if (1)
-    {
-        SoupSession* webkit_session = webkit_get_default_session ();
-
-        net = katze_net_new ();
-        s_session = katze_net_get_session (net);
-        g_type_set_qdata (SOUP_TYPE_SESSION,
-                          g_quark_from_static_string ("midori-app"), app);
-        katze_assign (config_file, build_config_filename ("cookies.txt"));
-        jar = soup_cookie_jar_new ();
-        soup_session_add_feature (s_session, SOUP_SESSION_FEATURE (jar));
-        soup_session_add_feature (webkit_session, SOUP_SESSION_FEATURE (jar));
-        g_object_unref (jar);
-        midori_soup_session_prepare (s_session, settings);
-        midori_soup_session_prepare (webkit_session, settings);
-        g_object_unref (net);
-    }
-
     /* Open as many tabs as we have uris, seperated by pipes */
     i = 0;
     while (uris && uris[i])
@@ -1919,13 +1668,10 @@ main (int    argc,
             G_CALLBACK (midori_search_engines_modify_cb), search_engines);
         if (!katze_array_is_empty (search_engines))
         {
-            guint n = katze_array_get_length (search_engines);
-            for (i = 0; i < n; i++)
-            {
-                item = katze_array_get_nth_item (search_engines, i);
+            i = 0;
+            while ((item = katze_array_get_nth_item (search_engines, i++)))
                 g_signal_connect_after (item, "notify",
                     G_CALLBACK (midori_search_engines_modify_cb), search_engines);
-            }
         }
     }
     katze_assign (config_file, build_config_filename ("bookmarks.xbel"));
@@ -1934,19 +1680,18 @@ main (int    argc,
         g_signal_connect_after (bookmarks, "add-item",
             G_CALLBACK (midori_bookmarks_add_item_cb), bookmarks);
         g_signal_connect_after (bookmarks, "remove-item",
-            G_CALLBACK (midori_bookmarks_remove_item_cb), NULL);
+            G_CALLBACK (midori_bookmarks_remove_item_cb), bookmarks);
         if (!katze_array_is_empty (bookmarks))
         {
-            guint n = katze_array_get_length (bookmarks);
-            for (i = 0; i < n; i++)
+            i = 0;
+            while ((item = katze_array_get_nth_item (bookmarks, i++)))
             {
-                item = katze_array_get_nth_item (bookmarks, i);
                 if (KATZE_IS_ARRAY (item))
                 {
                     g_signal_connect_after (item, "add-item",
                         G_CALLBACK (midori_bookmarks_add_item_cb), bookmarks);
                     g_signal_connect_after (item, "remove-item",
-                        G_CALLBACK (midori_bookmarks_remove_item_cb), NULL);
+                        G_CALLBACK (midori_bookmarks_remove_item_cb), bookmarks);
                 }
                 g_signal_connect_after (item, "notify",
                     G_CALLBACK (midori_bookmarks_notify_item_cb), bookmarks);
@@ -2000,15 +1745,21 @@ main (int    argc,
     g_signal_connect (app, "add-browser",
         G_CALLBACK (midori_app_add_browser_cb), NULL);
 
+    g_idle_add (midori_load_cookie_jar, settings);
     g_idle_add (midori_load_extensions, app);
     katze_item_set_parent (KATZE_ITEM (_session), app);
     g_idle_add (midori_load_session, _session);
 
     gtk_main ();
 
+    #if HAVE_HILDON
+    osso_deinitialize (osso_context);
+    #endif
+
     #if HAVE_SQLITE
     settings = katze_object_get_object (app, "settings");
     g_object_get (settings, "maximum-history-age", &max_history_age, NULL);
+    g_object_unref (settings);
     midori_history_terminate (db, max_history_age);
     #endif
     g_object_unref (app);
