@@ -27,6 +27,7 @@ struct _MidoriLocationAction
     GtkAction parent_instance;
 
     gchar* uri;
+    KatzeArray* search_engines;
     gdouble progress;
     gchar* secondary_icon;
 
@@ -207,8 +208,9 @@ midori_location_action_set_model (MidoriLocationAction* location_action,
         entry = gtk_bin_get_child (GTK_BIN (location_entry));
 
         g_object_set (location_entry, "model", model, NULL);
-        g_object_set (gtk_entry_get_completion (GTK_ENTRY (entry)),
-            "model", model ? location_action->sort_model : NULL, NULL);
+        gtk_entry_completion_set_model (
+            gtk_entry_get_completion (GTK_ENTRY (entry)),
+            model ? location_action->filter_model : NULL);
     }
 }
 
@@ -292,6 +294,7 @@ static void
 midori_location_action_init (MidoriLocationAction* location_action)
 {
     location_action->uri = NULL;
+    location_action->search_engines = NULL;
     location_action->progress = 0.0;
     location_action->secondary_icon = NULL;
     location_action->default_icon = NULL;
@@ -314,6 +317,7 @@ midori_location_action_finalize (GObject* object)
     MidoriLocationAction* location_action = MIDORI_LOCATION_ACTION (object);
 
     katze_assign (location_action->uri, NULL);
+    katze_assign (location_action->search_engines, NULL);
 
     katze_object_assign (location_action->model, NULL);
     katze_object_assign (location_action->sort_model, NULL);
@@ -498,42 +502,50 @@ midori_location_entry_render_text_cb (GtkCellLayout*   layout,
     gchar* skey;
     gchar* temp;
     gchar** parts;
+    size_t len;
 
     gtk_tree_model_get (model, iter, URI_COL, &uri, TITLE_COL, &title, -1);
 
     desc_uri = desc_title = key = NULL;
-    if (data)
+    if (G_LIKELY (data))
     {
         entry = gtk_entry_completion_get_entry (GTK_ENTRY_COMPLETION (data));
-        key = g_utf8_strdown (gtk_entry_get_text (GTK_ENTRY (entry)), -1);
+        key = title ? g_utf8_strdown (gtk_entry_get_text (GTK_ENTRY (entry)), -1)
+            : g_ascii_strdown (gtk_entry_get_text (GTK_ENTRY (entry)), -1);
+        len = 0;
     }
-    if (data && uri)
+    if (G_LIKELY (data && uri))
     {
-        temp = g_utf8_strdown (uri, -1);
-        start = strstr (temp, key);
-        if (start)
+        temp = g_ascii_strdown (uri, -1);
+        if ((start = strstr (temp, key)))
         {
-            skey = g_malloc0 (strlen (key) + 1);
-            g_utf8_strncpy (skey, uri + (start - temp), g_utf8_strlen (key, -1));
-            parts = g_strsplit (uri, skey, 2);
-            if (parts && parts[0] && parts[1])
-                desc_uri = g_markup_printf_escaped ("%s<b>%s</b>%s",
-                    parts[0], skey, parts[1]);
-            g_strfreev (parts);
+            len = strlen (key);
+            skey = g_malloc0 (len + 1);
+            strncpy (skey, uri + (start - temp), len);
+            if (skey && *skey && (parts = g_strsplit (uri, skey, 2)))
+            {
+                if (parts[0] && parts[1])
+                {
+                    desc_uri = g_markup_printf_escaped ("%s<b>%s</b>%s",
+                        parts[0], skey, parts[1]);
+                    g_strfreev (parts);
+                }
+            }
             g_free (skey);
         }
         g_free (temp);
     }
     if (uri && !desc_uri)
         desc_uri = g_markup_escape_text (uri, -1);
-    if (data && title)
+    if (G_LIKELY (data && title))
     {
         temp = g_utf8_strdown (title, -1);
-        start = strstr (temp, key);
-        if (start)
+        if ((start = strstr (temp, key)))
         {
-            skey = g_malloc0 (strlen (key) + 1);
-            g_utf8_strncpy (skey, title + (start - temp), g_utf8_strlen (key, -1));
+            if (!len)
+                len = strlen (key);
+            skey = g_malloc0 (len + 1);
+            strncpy (skey, title + (start - temp), len);
             parts = g_strsplit (title, skey, 2);
             if (parts && parts[0] && parts[1])
                 desc_title = g_markup_printf_escaped ("%s<b>%s</b>%s",
@@ -547,10 +559,14 @@ midori_location_entry_render_text_cb (GtkCellLayout*   layout,
         desc_title = g_markup_escape_text (title, -1);
 
     if (desc_title)
+    {
         desc = g_strdup_printf ("%s\n<span color='gray45'>%s</span>",
                                 desc_title, desc_uri);
+        g_free (desc_uri);
+        g_free (desc_title);
+    }
     else
-        desc = g_strdup_printf ("%s", desc_uri);
+        desc = desc_uri;
 
     g_object_set (renderer, "markup", desc,
         "ellipsize-set", TRUE, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
@@ -559,8 +575,6 @@ midori_location_entry_render_text_cb (GtkCellLayout*   layout,
     g_free (title);
     g_free (key);
     g_free (desc);
-    g_free (desc_uri);
-    g_free (desc_title);
 }
 
 static gboolean
@@ -579,14 +593,14 @@ midori_location_entry_completion_match_cb (GtkEntryCompletion* completion,
     gtk_tree_model_get (model, iter, URI_COL, &uri, TITLE_COL, &title, -1);
 
     match = FALSE;
-    if (uri)
+    if (G_LIKELY (uri))
     {
         temp = g_utf8_casefold (uri, -1);
         match = (strstr (temp, key) != NULL);
         g_free (temp);
         g_free (uri);
 
-        if (!match && title)
+        if (!match && G_LIKELY (title))
         {
             temp = g_utf8_casefold (title, -1);
             match = (strstr (temp, key) != NULL);
@@ -621,20 +635,6 @@ midori_location_action_set_item (MidoriLocationAction*    location_action,
             FAVICON_COL, new_icon, -1);
 }
 
-static gchar*
-midori_location_action_format_uri (const gchar* uri)
-{
-    gchar* new_uri;
-    size_t len;
-
-    new_uri = g_ascii_strdown (uri, -1);
-    len = strlen (new_uri);
-
-    if (new_uri [len - 1] == '/')
-        new_uri [len - 1] = '\0';
-    return new_uri;
-}
-
 /**
  * midori_location_action_iter_lookup:
  * @location_action: a #MidoriLocationAction
@@ -653,46 +653,34 @@ midori_location_action_iter_lookup (MidoriLocationAction* location_action,
 {
     GtkTreeModel* model;
     gchar* path;
-    gchar* new_uri;
-    gchar* tmp_uri;
-    gboolean found;
 
     model = location_action->model;
-    found = FALSE;
-
-    new_uri = midori_location_action_format_uri (uri);
 
     if (midori_location_action_is_frozen (location_action))
     {
-        if ((path = g_hash_table_lookup (location_action->items, new_uri)))
-        {
+        gboolean found = FALSE;
+        if ((path = g_hash_table_lookup (location_action->items, uri)))
             if (!(found = gtk_tree_model_get_iter_from_string (model, iter, path)))
-            {
-                g_hash_table_remove (location_action->items, new_uri);
-            }
-        }
+                g_hash_table_remove (location_action->items, uri);
+        return found;
     }
-    else
+
+    if (gtk_tree_model_get_iter_first (model, iter))
     {
-        if (gtk_tree_model_get_iter_first (model, iter))
+        gchar* tmp_uri = NULL;
+        do
         {
-            tmp_uri = NULL;
-            do
-            {
-
-                gtk_tree_model_get (model, iter, URI_COL, &tmp_uri, -1);
-                found = !g_ascii_strncasecmp (new_uri, tmp_uri, strlen (new_uri));
-                katze_assign (tmp_uri, NULL);
-
-                if (found)
-                    break;
-            }
-            while (gtk_tree_model_iter_next (model, iter));
+            gint cmp;
+            gtk_tree_model_get (model, iter, URI_COL, &tmp_uri, -1);
+            cmp = strcmp (uri, tmp_uri);
+            g_free (tmp_uri);
+            if (!cmp)
+                return TRUE;
         }
+        while (gtk_tree_model_iter_next (model, iter));
     }
-    g_free (new_uri);
 
-    return found;
+    return FALSE;
 }
 
 /**
@@ -716,15 +704,13 @@ midori_location_action_iter_insert (MidoriLocationAction* location_action,
     if (!midori_location_action_iter_lookup (location_action, uri, iter))
     {
         GtkTreeModel* model;
-        gchar* path;
-        gchar* new_uri;
 
         model = location_action->model;
         gtk_list_store_insert (GTK_LIST_STORE (model), iter, position);
         if (midori_location_action_is_frozen (location_action))
         {
-            new_uri = midori_location_action_format_uri (uri);
-            path = gtk_tree_model_get_string_from_iter (model,  iter);
+            gchar* new_uri = g_strdup (uri);
+            gchar* path = gtk_tree_model_get_string_from_iter (model,  iter);
             g_hash_table_insert (location_action->items, new_uri, path);
         }
         return FALSE;
@@ -749,6 +735,51 @@ midori_location_entry_match_selected_cb (GtkEntryCompletion*   completion,
     return FALSE;
 }
 
+static void
+midori_location_entry_action_activated_cb (GtkEntryCompletion*   completion,
+                                           gint                  action,
+                                           MidoriLocationAction* location_action)
+{
+    if (location_action->search_engines)
+    {
+        KatzeItem* item = katze_array_get_nth_item (
+            location_action->search_engines, action);
+        GtkWidget* entry = gtk_entry_completion_get_entry (completion);
+        const gchar* keywords = gtk_entry_get_text (GTK_ENTRY (entry));
+        const gchar* uri = katze_item_get_uri (item);
+        gchar* search;
+        if (!item)
+            return;
+        if (strstr (uri, "%s"))
+            search = g_strdup_printf (uri, keywords);
+        else
+            search = g_strconcat (uri, " ", keywords, NULL);
+        midori_location_action_set_uri (location_action, search);
+        g_signal_emit (location_action, signals[SUBMIT_URI], 0, search, FALSE);
+        g_free (search);
+    }
+}
+
+static void
+midori_location_action_add_actions (GtkEntryCompletion* completion,
+                                    KatzeArray*         search_engines)
+{
+    guint i;
+    KatzeItem* item;
+
+    if (!search_engines)
+        return;
+
+    i = 0;
+    while ((item = katze_array_get_nth_item (search_engines, i)))
+    {
+        gchar* text = g_strdup_printf (_("Search with %s"),
+            katze_item_get_name (item));
+        gtk_entry_completion_insert_action_text (completion, i++, text);
+        g_free (text);
+    }
+}
+
 static void
 midori_location_action_completion_init (MidoriLocationAction* location_action,
                                         GtkWidget*            location_entry)
@@ -784,6 +815,11 @@ midori_location_action_completion_init (MidoriLocationAction* location_action,
     g_signal_connect (completion, "match-selected",
         G_CALLBACK (midori_location_entry_match_selected_cb), location_action);
 
+    midori_location_action_add_actions (completion,
+                                        location_action->search_engines);
+    g_signal_connect (completion, "action-activated",
+        G_CALLBACK (midori_location_entry_action_activated_cb), location_action);
+
     g_object_unref (completion);
 }
 
@@ -868,7 +904,7 @@ midori_location_action_connect_proxy (GtkAction* action,
                       midori_location_action_key_press_event_cb, action,
                       "signal::focus-out-event",
                       midori_location_action_focus_out_event_cb, action,
-                      "signal::icon-released",
+                      "signal::icon-release",
                       midori_location_action_icon_released_cb, action,
                       NULL);
     }
@@ -1172,6 +1208,55 @@ midori_location_action_set_title_for_uri (MidoriLocationAction* location_action,
     midori_location_action_prepend_item (location_action, &item);
 }
 
+/**
+ * midori_location_action_set_search_engines:
+ * @location_action: a #MidoriLocationAction
+ * @search_engines: a #KatzeArray
+ *
+ * Assigns the specified search engines to the location action.
+ * Search engines will appear as actions in the completion.
+ *
+ * Since: 0.1.6
+ **/
+void
+midori_location_action_set_search_engines (MidoriLocationAction* location_action,
+                                           KatzeArray*           search_engines)
+{
+    GSList* proxies;
+    GtkWidget* alignment;
+    GtkWidget* entry;
+    GtkWidget* child;
+    GtkEntryCompletion* completion;
+
+    g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
+
+    if (search_engines)
+        g_object_ref (search_engines);
+
+    proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
+
+    for (; proxies != NULL; proxies = g_slist_next (proxies))
+    if (GTK_IS_TOOL_ITEM (proxies->data))
+    {
+        KatzeItem* item;
+        guint i;
+
+        alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
+        entry = gtk_bin_get_child (GTK_BIN (alignment));
+        child = gtk_bin_get_child (GTK_BIN (entry));
+
+        completion = gtk_entry_get_completion (GTK_ENTRY (child));
+        i = 0;
+        if (location_action->search_engines)
+        while ((item = katze_array_get_nth_item (location_action->search_engines, i++)))
+            gtk_entry_completion_delete_action (completion, 0);
+        midori_location_action_add_actions (completion, search_engines);
+    }
+
+    katze_object_assign (location_action->search_engines, search_engines);
+    /* FIXME: Take care of adding and removing search engines as needed */
+}
+
 gdouble
 midori_location_action_get_progress (MidoriLocationAction* location_action)
 {
@@ -1239,8 +1324,8 @@ midori_location_action_set_secondary_icon (MidoriLocationAction* location_action
 }
 
 /**
- * midori_location_entry_set_item_from_uri:
- * @location_entry: a #MidoriLocationEntry
+ * midori_location_action_set_item_from_uri:
+ * @location_action: a #MidoriLocationAction
  * @uri: a string
  *
  * Finds the item from the list matching @uri
