@@ -418,16 +418,16 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
         gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
     }
     FILLED_ADD (hbox, 1, 2, 1, 2);
-    button = katze_property_proxy (settings, "show-crash-dialog", "blurb");
+    button = katze_property_proxy (settings, "show-crash-dialog", NULL);
     SPANNED_ADD (button, 0, 2, 2, 3);
     FRAME_NEW (_("Transfers"));
-    TABLE_NEW (3, 2);
+    TABLE_NEW (4, 2);
     #if WEBKIT_CHECK_VERSION (1, 1, 3)
     label = katze_property_label (settings, "download-folder");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "download-folder", "folder");
     FILLED_ADD (button, 1, 2, 0, 1);
-    #else
+    #endif
     label = katze_property_label (settings, "download-manager");
     INDENTED_ADD (label, 0, 1, 1, 2);
     hbox = gtk_hbox_new (FALSE, 4);
@@ -442,7 +442,6 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_signal_connect (entry, "focus-out-event",
         G_CALLBACK (proxy_download_manager_icon_cb), button);
     FILLED_ADD (hbox, 1, 2, 1, 2);
-    #endif
     label = katze_property_label (settings, "text-editor");
     INDENTED_ADD (label, 0, 1, 2, 3);
     hbox = gtk_hbox_new (FALSE, 4);
@@ -457,6 +456,20 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     g_signal_connect (entry, "focus-out-event",
         G_CALLBACK (proxy_download_manager_icon_cb), button);
     FILLED_ADD (hbox, 1, 2, 2, 3);
+    label = katze_property_label (settings, "news-aggregator");
+    INDENTED_ADD (label, 0, 1, 3, 4);
+    hbox = gtk_hbox_new (FALSE, 4);
+    button = gtk_image_new ();
+    gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
+        GTK_ICON_SIZE_MENU, &icon_width, &icon_height);
+    gtk_widget_set_size_request (button, icon_width, icon_height);
+    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 4);
+    entry = katze_property_proxy (settings, "news-aggregator", NULL);
+    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+    proxy_download_manager_icon_cb (entry, NULL, GTK_IMAGE (button));
+    g_signal_connect (entry, "focus-out-event",
+        G_CALLBACK (proxy_download_manager_icon_cb), button);
+    FILLED_ADD (hbox, 1, 2, 3, 4);
 
     /* Page "Appearance" */
     PAGE_NEW (GTK_STOCK_SELECT_FONT, _("Appearance"));
@@ -493,7 +506,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     /* Page "Behavior" */
     PAGE_NEW (GTK_STOCK_SELECT_COLOR, _("Behavior"));
     FRAME_NEW (_("Features"));
-    TABLE_NEW (7, 2);
+    TABLE_NEW (6, 2);
     button = katze_property_proxy (settings, "auto-load-images", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Load images automatically"));
     gtk_widget_set_tooltip_text (button, _("Load and display images automatically"));
@@ -530,10 +543,6 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     SPANNED_ADD (button, 0, 1, 4, 5);
     button = katze_property_proxy (settings, "find-while-typing", NULL);
     SPANNED_ADD (button, 1, 2, 4, 5);
-    label = katze_property_label (settings, "location-entry-search");
-    INDENTED_ADD (label, 0, 1, 5, 6);
-    entry = katze_property_proxy (settings, "location-entry-search", NULL);
-    FILLED_ADD (entry, 1, 2, 5, 6);
 
     /* Page "Interface" */
     PAGE_NEW (GTK_STOCK_CONVERT, _("Interface"));
@@ -545,6 +554,8 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     FILLED_ADD (button, 1, 2, 0, 1);
     #endif
     button = katze_property_proxy (settings, "progress-in-location", NULL);
+    FILLED_ADD (button, 0, 1, 1, 2);
+    button = katze_property_proxy (settings, "search-engines-in-completion", NULL);
     FILLED_ADD (button, 1, 2, 1, 2);
     FRAME_NEW (_("Browsing"));
     TABLE_NEW (5, 2);
@@ -616,7 +627,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "accept-cookies", NULL);
     FILLED_ADD (button, 1, 2, 0, 1);
-    button = katze_property_proxy (settings, "original-cookies-only", "blurb");
+    button = katze_property_proxy (settings, "original-cookies-only", NULL);
     SPANNED_ADD (button, 0, 2, 1, 2);
     label = katze_property_label (settings, "maximum-cookie-age");
     INDENTED_ADD (label, 0, 1, 2, 3);
