@@ -39,6 +39,11 @@ webkit_web_view_get_selected_text (WebKitWebView* web_view);
 void
 webkit_web_frame_print (WebKitWebFrame* web_frame);
 
+GdkPixbuf*
+midori_search_action_get_icon (KatzeNet*  net,
+                               KatzeItem* item,
+                               GtkWidget* widget);
+
 struct _MidoriView
 {
     GtkScrolledWindow parent_instance;
@@ -58,6 +63,7 @@ struct _MidoriView
     /* KatzeArray* news_feeds; */
 
     gchar* download_manager;
+    gchar* news_aggregator;
     gboolean middle_click_opens_selection;
     gboolean open_tabs_in_the_background;
     gboolean close_buttons_on_tabs;
@@ -617,31 +623,6 @@ webkit_web_view_progress_changed_cb (WebKitWebView* web_view,
     g_object_notify (G_OBJECT (view), "progress");
 }
 
-/*static void
-gjs_value_links_foreach_cb (GjsValue*   link,
-                            MidoriView* view)
-{
-    const gchar* type;
-
-    if (gjs_value_is_object (link) && gjs_value_has_attribute (link, "href"))
-    {
-        if (gjs_value_has_attribute (link, "type"))
-        {
-            type = gjs_value_get_attribute_string (link, "type");
-            if (!strcmp (type, "application/rss+xml")
-                || !strcmp (type, "application/x.atom+xml")
-                || !strcmp (type, "application/atom+xml"))
-            {
-                katze_array_add_item (view->news_feeds, link);
-                g_signal_emit (view, signals[NEWS_FEED_READY],
-                    gjs_value_get_attribute_string (link, "href"), type,
-                    gjs_value_has_attribute (link, "title")
-                    ? gjs_value_get_attribute_string (link, "title") : NULL);
-            }
-        }
-    }
-}*/
-
 static void
 webkit_web_frame_load_done_cb (WebKitWebFrame* web_frame,
                                gboolean        success,
@@ -654,6 +635,7 @@ webkit_web_frame_load_done_cb (WebKitWebFrame* web_frame,
     {
         /* i18n: The title of the 404 - Not found error page */
         title = g_strdup_printf (_("Not found - %s"), view->uri);
+        katze_assign (view->title, title);
         data = g_strdup_printf (
             "<html><head><title>%s</title></head>"
             "<body><h1>%s</h1>"
@@ -664,7 +646,6 @@ webkit_web_frame_load_done_cb (WebKitWebFrame* web_frame,
             "or move on to another page."
             "</body></html>",
             title, title, view->uri);
-        g_free (title);
         webkit_web_view_load_html_string (
             WEBKIT_WEB_VIEW (view->web_view), data, view->uri);
         g_free (data);
@@ -678,28 +659,28 @@ webkit_web_view_load_finished_cb (WebKitWebView*  web_view,
                                   WebKitWebFrame* web_frame,
                                   MidoriView*     view)
 {
-    /* JSContextRef js_context;
-    GjsValue* value;
-    GjsValue* document;
-    GjsValue* links; */
-
     g_object_freeze_notify (G_OBJECT (view));
 
     view->progress = 1.0;
     g_object_notify (G_OBJECT (view), "progress");
     midori_view_update_load_status (view, MIDORI_LOAD_FINISHED);
 
-    g_object_thaw_notify (G_OBJECT (view));
+    if (view->news_aggregator && *view->news_aggregator)
+    {
+        JSContextRef js_context = webkit_web_frame_get_global_context (web_frame);
+        gchar* value = sokoke_js_script_eval (js_context,
+        "function feeds (l) { var f = new Array (); for (i in l) "
+        "{ var t = l[i].type; "
+        "if (t && (t.indexOf ('rss') != -1 || t.indexOf ('atom') != -1)) "
+        "f.push (l[i].href); } return f; }"
+        "feeds (document.getElementsByTagName ('link'))", NULL);
+        g_object_set_data (G_OBJECT (view), "news-feeds",
+                           value && *value ? (void*)1 : (void*)0);
+        /* Ensure load-status is notified again, whether it changed or not */
+        g_object_notify (G_OBJECT (view), "load-status");
+    }
 
-    /* js_context = webkit_web_frame_get_global_context (web_frame);
-    value = gjs_value_new (js_context, NULL);
-    document = gjs_value_get_by_name (value, "document");
-    links = gjs_value_get_elements_by_tag_name (document, "link");
-    katze_array_clear (view->news_feeds);
-    gjs_value_foreach (links, (GjsCallback)gjs_value_links_foreach_cb, view);
-    g_object_unref (links);
-    g_object_unref (document);
-    g_object_unref (value); */
+    g_object_thaw_notify (G_OBJECT (view));
 }
 
 static void
@@ -762,13 +743,13 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
             background = view->open_tabs_in_the_background;
             if (event->state & GDK_SHIFT_MASK)
                 background = !background;
-            g_signal_emit_by_name (view, "new-tab", link_uri, background);
+            g_signal_emit (view, signals[NEW_TAB], 0, link_uri, background);
             return TRUE;
         }
         else if (event->state & GDK_SHIFT_MASK)
         {
             /* Open link in new window */
-            g_signal_emit_by_name (view, "new-window", link_uri);
+            g_signal_emit (view, signals[NEW_WINDOW], 0, link_uri);
             return TRUE;
         }
         else if (event->state & GDK_MOD1_MASK)
@@ -777,7 +758,7 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
             background = view->open_tabs_in_the_background;
             if (event->state & GDK_CONTROL_MASK)
                 background = !background;
-            g_signal_emit_by_name (view, "new-tab", link_uri, background);
+            g_signal_emit (view, signals[NEW_TAB], 0, link_uri, background);
             return TRUE;
         }
         break;
@@ -788,7 +769,7 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
             background = view->open_tabs_in_the_background;
             if (event->state & GDK_CONTROL_MASK)
                 background = !background;
-            g_signal_emit_by_name (view, "new-tab", link_uri, background);
+            g_signal_emit (view, signals[NEW_TAB], 0, link_uri, background);
             return TRUE;
         }
         else if (event->state & GDK_CONTROL_MASK)
@@ -798,19 +779,23 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
         }
         else if (view->middle_click_opens_selection)
         {
+            guint i = 0;
             clipboard = gtk_clipboard_get_for_display (
                 gtk_widget_get_display (GTK_WIDGET (view)),
                 GDK_SELECTION_PRIMARY);
             uri = gtk_clipboard_wait_for_text (clipboard);
-            if (uri && strchr (uri, '.') && !strchr (g_strstrip (uri), ' '))
+            if (uri && strchr (uri, '.'))
             {
-                new_uri = sokoke_magic_uri (uri, NULL);
+                while (uri[i++] != '\0')
+                    if (uri[i] == '\n' || uri[i] == '\r')
+                        uri[i] = ' ';
+                new_uri = sokoke_magic_uri (g_strstrip (uri), NULL);
                 if (event->state & GDK_CONTROL_MASK)
                 {
                     background = view->open_tabs_in_the_background;
                     if (event->state & GDK_CONTROL_MASK)
                         background = !background;
-                    g_signal_emit_by_name (view, "new-tab", new_uri, background);
+                    g_signal_emit (view, signals[NEW_TAB], 0, new_uri, background);
                 }
                 else
                 {
@@ -818,9 +803,9 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
                     gtk_widget_grab_focus (GTK_WIDGET (view));
                 }
                 g_free (new_uri);
-                g_free (uri);
                 return TRUE;
             }
+            g_free (uri);
         }
         break;
     case 8:
@@ -866,9 +851,11 @@ gtk_widget_scroll_event_cb (WebKitWebView*  web_view,
     if (event->state & GDK_CONTROL_MASK)
     {
         if (event->direction == GDK_SCROLL_DOWN)
-            webkit_web_view_zoom_out (WEBKIT_WEB_VIEW (web_view));
+            midori_view_set_zoom_level (view,
+                midori_view_get_zoom_level (view) - 0.25f);
         else if(event->direction == GDK_SCROLL_UP)
-            webkit_web_view_zoom_in (WEBKIT_WEB_VIEW (web_view));
+            midori_view_set_zoom_level (view,
+                midori_view_get_zoom_level (view) + 0.25f);
         return TRUE;
     }
     else
@@ -898,18 +885,19 @@ static void
 midori_web_view_menu_search_web_activate_cb (GtkWidget*  widget,
                                              MidoriView* view)
 {
+    gchar* search;
     gchar* uri;
-    gchar* location_entry_search;
 
-    g_object_get (view->settings, "location-entry-search",
-                  &location_entry_search, NULL);
-    if (strstr (location_entry_search, "%s"))
-    {
-        uri = g_strdup_printf (location_entry_search, view->selected_text);
-        g_free (location_entry_search);
-    }
+    if ((search = g_object_get_data (G_OBJECT (widget), "search")))
+        search = g_strdup (search);
     else
-        uri = location_entry_search;
+        g_object_get (view->settings, "location-entry-search",
+                      &search, NULL);
+    if (strstr (search, "%s"))
+        uri = g_strdup_printf (search, view->selected_text);
+    else
+        uri = g_strconcat (search, view->selected_text, NULL);
+    g_free (search);
 
     g_signal_emit (view, signals[NEW_TAB], 0, uri,
         view->open_tabs_in_the_background);
@@ -924,6 +912,7 @@ midori_web_view_menu_save_as_activate_cb (GtkWidget*  widget,
 {
     g_signal_emit (view, signals[SAVE_AS], 0, view->link_uri);
 }
+#endif
 
 static void
 midori_web_view_menu_download_activate_cb (GtkWidget*  widget,
@@ -931,7 +920,6 @@ midori_web_view_menu_download_activate_cb (GtkWidget*  widget,
 {
     sokoke_spawn_program (view->download_manager, view->link_uri);
 }
-#endif
 
 static void
 midori_web_view_menu_add_bookmark_activate_cb (GtkWidget*  widget,
@@ -1024,10 +1012,11 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
         g_signal_connect (menuitem, "activate",
             G_CALLBACK (midori_web_view_menu_save_as_activate_cb), view);
         gtk_widget_show (menuitem);
+        #endif
         if (view->download_manager && *view->download_manager)
         {
             menuitem = gtk_image_menu_item_new_with_mnemonic (
-                _("_Download Link destination"));
+                _("Download with Download _Manager"));
             icon = gtk_image_new_from_stock (GTK_STOCK_SAVE_AS,
                                              GTK_ICON_SIZE_MENU);
             gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
@@ -1036,7 +1025,6 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
                 G_CALLBACK (midori_web_view_menu_download_activate_cb), view);
             gtk_widget_show (menuitem);
         }
-        #endif
         menuitem = gtk_image_menu_item_new_from_stock (STOCK_BOOKMARK_ADD, NULL);
         gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menuitem, 5);
         g_signal_connect (menuitem, "activate",
@@ -1046,6 +1034,41 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
 
     if (!view->link_uri && has_selection)
     {
+        GtkWidget* window;
+        guint i;
+
+        window = gtk_widget_get_toplevel (GTK_WIDGET (web_view));
+        i = 0;
+        if (katze_object_has_property (window, "search-engines"))
+        {
+            KatzeArray* search_engines;
+            KatzeItem* item;
+            GtkWidget* sub_menu = gtk_menu_new ();
+
+            menuitem = gtk_image_menu_item_new_with_mnemonic (_("Search _with"));
+            gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), sub_menu);
+            gtk_menu_shell_insert (GTK_MENU_SHELL (menu), menuitem, 1);
+            gtk_widget_show (menuitem);
+
+            search_engines = katze_object_get_object (window, "search-engines");
+            while ((item = katze_array_get_nth_item (search_engines, i++)))
+            {
+                GdkPixbuf* pixbuf;
+                menuitem = gtk_image_menu_item_new_with_mnemonic (katze_item_get_name (item));
+                pixbuf = midori_search_action_get_icon (view->net, item,
+                                                        GTK_WIDGET (web_view));
+                icon = gtk_image_new_from_pixbuf (pixbuf);
+                g_object_unref (pixbuf);
+                gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
+                gtk_menu_shell_insert (GTK_MENU_SHELL (sub_menu), menuitem, i - 1);
+                g_object_set_data (G_OBJECT (menuitem), "search",
+                                   (gchar*)katze_item_get_uri (item));
+                g_signal_connect (menuitem, "activate",
+                    G_CALLBACK (midori_web_view_menu_search_web_activate_cb), view);
+                gtk_widget_show (menuitem);
+            }
+            g_object_unref (search_engines);
+        }
         items = gtk_container_get_children (GTK_CONTAINER (menu));
         menuitem = (GtkWidget*)g_list_nth_data (items, 0);
         /* hack to localize menu item */
@@ -1055,6 +1078,7 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
         g_signal_connect (menuitem, "activate",
             G_CALLBACK (midori_web_view_menu_search_web_activate_cb), view);
         g_list_free (items);
+
         if (strchr (view->selected_text, '.')
             && !strchr (view->selected_text, ' '))
         {
@@ -1077,8 +1101,11 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
         items = gtk_container_get_children (GTK_CONTAINER (menu));
         menuitem = (GtkWidget*)g_list_nth_data (items, 3);
         /* hack to localize menu item */
-        label = gtk_bin_get_child (GTK_BIN (menuitem));
-        gtk_label_set_label (GTK_LABEL (label), D_("gtk20", "_Refresh"));
+        if (GTK_IS_BIN (menuitem))
+        {
+            label = gtk_bin_get_child (GTK_BIN (menuitem));
+            gtk_label_set_label (GTK_LABEL (label), D_("gtk20", "_Refresh"));
+        }
         g_list_free (items);
         menuitem = gtk_image_menu_item_new_with_mnemonic (_("Undo Close Tab"));
         icon = gtk_image_new_from_stock (GTK_STOCK_UNDELETE, GTK_ICON_SIZE_MENU);
@@ -1178,10 +1205,16 @@ webkit_web_view_mime_type_decision_cb (GtkWidget*               web_view,
     GtkWidget* dialog;
     gchar* content_type;
     gchar* description;
+    #if GTK_CHECK_VERSION (2, 14, 0)
+    GIcon* icon;
+    GtkWidget* image;
+    #endif
     gchar* title;
     GdkScreen* screen;
     GtkIconTheme* icon_theme;
     gint response;
+    #else
+    gchar* uri;
     #endif
 
     if (web_frame != webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (web_view)))
@@ -1205,9 +1238,20 @@ webkit_web_view_mime_type_decision_cb (GtkWidget*               web_view,
     content_type = g_strdup (mime_type);
     #endif
     description = g_content_type_get_description (content_type);
+    #if GTK_CHECK_VERSION (2, 14, 0)
+    icon = g_content_type_get_icon (content_type);
+    image = gtk_image_new_from_gicon (icon, GTK_ICON_SIZE_DIALOG);
+    g_object_unref (icon);
+    gtk_widget_show (image);
+    gtk_message_dialog_set_image (GTK_MESSAGE_DIALOG (dialog), image);
+    #endif
     g_free (content_type);
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-        _("File Type: %s ('%s')"), description, mime_type);
+    if (g_strrstr (description, mime_type))
+        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+        _("File Type: '%s'"), mime_type);
+    else
+       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+       _("File Type: %s ('%s')"), description, mime_type);
     g_free (description);
     gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
     /* i18n: A file open dialog title, ie. "Open http://fila.com/manual.tgz" */
@@ -1314,6 +1358,7 @@ midori_view_init (MidoriView* view)
     view->item = NULL;
 
     view->download_manager = NULL;
+    view->news_aggregator = NULL;
     view->web_view = NULL;
 
     /* Adjustments are not created automatically */
@@ -1342,6 +1387,7 @@ midori_view_finalize (GObject* object)
     katze_object_assign (view->item, NULL);
 
     katze_assign (view->download_manager, NULL);
+    katze_assign (view->news_aggregator, NULL);
 
     katze_object_assign (view->net, NULL);
 
@@ -1464,6 +1510,7 @@ _midori_view_update_settings (MidoriView* view)
 
     g_object_get (view->settings,
         "download-manager", &view->download_manager,
+        "news-aggregator", &view->news_aggregator,
         "zoom-text-and-images", &zoom_text_and_images,
         "close-buttons-on-tabs", &view->close_buttons_on_tabs,
         "open-new-pages-in", &view->open_new_pages_in,
@@ -1493,6 +1540,10 @@ midori_view_settings_notify_cb (MidoriWebSettings* settings,
     {
         katze_assign (view->download_manager, g_value_dup_string (&value));
     }
+    else if (name == g_intern_string ("news-aggregator"))
+    {
+        katze_assign (view->news_aggregator, g_value_dup_string (&value));
+    }
     else if (name == g_intern_string ("zoom-text-and-images"))
     {
         if (view->web_view)
@@ -1797,6 +1848,16 @@ midori_view_set_uri (MidoriView*  view,
         {
             midori_view_execute_script (view, &uri[11], NULL);
         }
+        else if (g_str_has_prefix (uri, "mailto:"))
+        {
+            if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL))
+            {
+                /* Fallback to Exo for example if GConf isn't setup */
+                gchar* command = g_strconcat ("exo-open ", uri, NULL);
+                g_spawn_command_line_async (command, NULL);
+                g_free (command);
+            }
+        }
         else
         {
             katze_assign (view->uri, g_strdup (uri));
@@ -2368,6 +2429,8 @@ midori_view_can_view_source (MidoriView* view)
         && !g_strrstr (view->mime_type, "xml"))
         return FALSE;
 
+    if (!uri)
+        return FALSE;
     if (g_str_has_prefix (uri, "http://") || g_str_has_prefix (uri, "https://"))
         return TRUE;
     if (g_str_has_prefix (uri, "file://"))
@@ -2401,12 +2464,21 @@ void
 midori_view_reload (MidoriView* view,
                     gboolean    from_cache)
 {
+    gchar* title;
+
     g_return_if_fail (MIDORI_IS_VIEW (view));
 
-    if (from_cache)
+    /* Error pages are special, we want to try loading the destination
+       again, not the error page which isn't even a proper page */
+    title = g_strdup_printf (_("Not found - %s"), view->uri);
+    if (view->title && strstr (title, view->title))
+        webkit_web_view_open (WEBKIT_WEB_VIEW (view->web_view), view->uri);
+    else if (from_cache)
         webkit_web_view_reload (WEBKIT_WEB_VIEW (view->web_view));
     else
         webkit_web_view_reload_bypass_cache (WEBKIT_WEB_VIEW (view->web_view));
+
+    g_free (title);
 }
 
 /**
