@@ -41,6 +41,7 @@ struct _MidoriWebSettings
 
     MidoriToolbarStyle toolbar_style;
     gboolean progress_in_location;
+    gboolean search_engines_in_completion;
     gchar* toolbar_items;
     gboolean compact_sidepanel;
     gboolean right_align_sidepanel;
@@ -51,6 +52,7 @@ struct _MidoriWebSettings
     gchar* download_folder;
     gchar* download_manager;
     gchar* text_editor;
+    gchar* news_aggregator;
     gchar* location_entry_search;
     MidoriPreferredEncoding preferred_encoding;
 
@@ -109,6 +111,7 @@ enum
 
     PROP_TOOLBAR_STYLE,
     PROP_PROGRESS_IN_LOCATION,
+    PROP_SEARCH_ENGINES_IN_COMPLETION,
     PROP_TOOLBAR_ITEMS,
     PROP_COMPACT_SIDEPANEL,
     PROP_RIGHT_ALIGN_SIDEPANEL,
@@ -119,6 +122,7 @@ enum
     PROP_DOWNLOAD_FOLDER,
     PROP_DOWNLOAD_MANAGER,
     PROP_TEXT_EDITOR,
+    PROP_NEWS_AGGREGATOR,
     PROP_LOCATION_ENTRY_SEARCH,
     PROP_PREFERRED_ENCODING,
 
@@ -479,6 +483,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      TRUE,
                                      flags));
 
+    /**
+    * MidoriWebSettings:search-engines-in-completion:
+    *
+    * Whether to show search engines in the location completion.
+    *
+    * Since: 0.1.6
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_SEARCH_ENGINES_IN_COMPLETION,
+                                     g_param_spec_boolean (
+                                     "search-engines-in-completion",
+                                     _("Search engines in location completion"),
+                                     _("Whether to show search engines in the location completion"),
+                                     TRUE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_TOOLBAR_ITEMS,
                                      g_param_spec_string (
@@ -580,6 +600,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      NULL,
                                      flags));
 
+    /**
+    * MidoriWebSettings:news-aggregator:
+    *
+    * An external news aggregator.
+    *
+    * Since: 0.1.6
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_NEWS_AGGREGATOR,
+                                     g_param_spec_string (
+                                     "news-aggregator",
+                                     _("News Aggregator"),
+                                     _("An external news aggregator"),
+                                     NULL,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_LOCATION_ENTRY_SEARCH,
                                      g_param_spec_string (
@@ -1018,6 +1054,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_PROGRESS_IN_LOCATION:
         web_settings->progress_in_location = g_value_get_boolean (value);
         break;
+    case PROP_SEARCH_ENGINES_IN_COMPLETION:
+        web_settings->search_engines_in_completion = g_value_get_boolean (value);
+        break;
     case PROP_TOOLBAR_ITEMS:
         katze_assign (web_settings->toolbar_items, g_value_dup_string (value));
         break;
@@ -1046,6 +1085,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_TEXT_EDITOR:
         katze_assign (web_settings->text_editor, g_value_dup_string (value));
         break;
+    case PROP_NEWS_AGGREGATOR:
+        katze_assign (web_settings->news_aggregator, g_value_dup_string (value));
+        break;
     case PROP_LOCATION_ENTRY_SEARCH:
         katze_assign (web_settings->location_entry_search, g_value_dup_string (value));
         break;
@@ -1138,8 +1180,8 @@ midori_web_settings_set_property (GObject*      object,
         if (web_settings->identify_as != MIDORI_IDENT_CUSTOM)
         {
             gchar* string = generate_ident_string (web_settings->identify_as);
-            g_object_set (object, "ident-string", string, NULL);
-            g_free (string);
+            katze_assign (web_settings->ident_string, string);
+            g_object_notify (object, "ident-string");
         }
         break;
     case PROP_IDENT_STRING:
@@ -1211,6 +1253,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_PROGRESS_IN_LOCATION:
         g_value_set_boolean (value, web_settings->progress_in_location);
         break;
+    case PROP_SEARCH_ENGINES_IN_COMPLETION:
+        g_value_set_boolean (value, web_settings->search_engines_in_completion);
+        break;
     case PROP_TOOLBAR_ITEMS:
         g_value_set_string (value, web_settings->toolbar_items);
         break;
@@ -1239,6 +1284,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_TEXT_EDITOR:
         g_value_set_string (value, web_settings->text_editor);
         break;
+    case PROP_NEWS_AGGREGATOR:
+        g_value_set_string (value, web_settings->news_aggregator);
+        break;
     case PROP_LOCATION_ENTRY_SEARCH:
         g_value_set_string (value, web_settings->location_entry_search);
         break;
@@ -1316,6 +1364,11 @@ midori_web_settings_get_property (GObject*    object,
         g_value_set_enum (value, web_settings->identify_as);
         break;
     case PROP_IDENT_STRING:
+        if (!g_strcmp0 (web_settings->ident_string, ""))
+        {
+            gchar* string = generate_ident_string (web_settings->identify_as);
+            katze_assign (web_settings->ident_string, string);
+        }
         g_value_set_string (value, web_settings->ident_string);
         break;
     case PROP_CACHE_SIZE:
