@@ -25,6 +25,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
+#include <glib/gstdio.h>
 
 #if HAVE_LIBIDN
     #include <stringprep.h>
@@ -46,11 +47,14 @@ sokoke_js_string_utf8 (JSStringRef js_string)
     return string_utf8;
 }
 
-JSValueRef
+gchar*
 sokoke_js_script_eval (JSContextRef js_context,
                        const gchar* script,
                        gchar**      exception)
 {
+    gchar* value;
+    JSStringRef js_value_string;
+
     g_return_val_if_fail (js_context, FALSE);
     g_return_val_if_fail (script, FALSE);
 
@@ -67,7 +71,11 @@ sokoke_js_script_eval (JSContextRef js_context,
         js_value = JSValueMakeNull (js_context);
     }
     JSStringRelease (js_script);
-    return js_value;
+
+    js_value_string = JSValueToStringCopy (js_context, js_value, NULL);
+    value = sokoke_js_string_utf8 (js_value_string);
+    JSStringRelease (js_value_string);
+    return value;
 }
 
 static void
@@ -189,7 +197,7 @@ sokoke_idn_to_punycode (gchar* uri)
 
     if (proto)
     {
-        result = g_strdup_printf ("%s://%s%s", proto, s, path ? path : "");
+        result = g_strconcat (proto, "://", s, path ? path : "", NULL);
         g_free (proto);
         if (path)
             g_free (hostname);
@@ -220,8 +228,10 @@ sokoke_magic_uri (const gchar* uri,
     g_return_val_if_fail (!search_engines ||
         katze_array_is_a (search_engines, KATZE_TYPE_ITEM), NULL);
 
-    /* Just return if it's a javascript: uri */
-    if (g_str_has_prefix (uri, "javascript:"))
+    /* Just return if it's a javascript: or mailto: uri */
+    if (g_str_has_prefix (uri, "javascript:")
+     || g_str_has_prefix (uri, "mailto:")
+     || g_str_has_prefix (uri, "data:"))
         return g_strdup (uri);
     /* Add file:// if we have a local path */
     if (g_path_is_absolute (uri))
@@ -235,52 +245,50 @@ sokoke_magic_uri (const gchar* uri,
         g_free (current_dir);
         return result;
     }
-    /* Do we need to add a protocol? */
-    if (!strstr (uri, "://"))
+    /* Do we have a protocol? */
+    if (g_strstr_len (uri, 8, "://"))
+        return sokoke_idn_to_punycode (g_strdup (uri));
+
+    /* Do we have a domain, ip address or localhost? */
+    search = NULL;
+    if (!strchr (uri, ' ') &&
+        ((search = strchr (uri, ':')) || (search = strchr (uri, '@'))) &&
+        search[0] && !g_ascii_isalpha (search[1]))
+        return sokoke_idn_to_punycode (g_strconcat ("http://", uri, NULL));
+    if (!strcmp (uri, "localhost") || g_str_has_prefix (uri, "localhost/"))
+        return g_strconcat ("http://", uri, NULL);
+    parts = g_strsplit (uri, ".", 0);
+    if (!search && parts[0] && parts[1])
     {
-        /* Do we have a domain, ip address or localhost? */
-        search = strchr (uri, ':');
-        if (search && search[0] &&
-            !g_ascii_isalpha (search[1]) && search[1] != ' ')
-            if (!strchr (search, '.'))
-                return sokoke_idn_to_punycode (g_strconcat ("http://", uri, NULL));
-        if (!strcmp (uri, "localhost") || g_str_has_prefix (uri, "localhost/"))
-            return g_strconcat ("http://", uri, NULL);
-        parts = g_strsplit (uri, ".", 0);
-        if (!search && parts[0] && parts[1])
-        {
-            search = NULL;
-            if (!(parts[1][1] == '\0' && !g_ascii_isalpha (parts[1][0])))
-                if (!strchr (parts[0], ' ') && !strchr (parts[1], ' '))
-                    search = g_strconcat ("http://", uri, NULL);
-            g_free (parts);
-            if (search)
-                return sokoke_idn_to_punycode (search);
-        }
-        /* We don't want to search? So return early. */
-        if (!search_engines)
-            return g_strdup (uri);
-        search = NULL;
-        search_uri = NULL;
-        /* Do we have a keyword and a string? */
-        parts = g_strsplit (uri, " ", 2);
-        if (parts[0] && parts[1])
-        {
-            item = katze_array_find_token (search_engines, parts[0]);
-            if (item)
-                search_uri = katze_item_get_uri (item);
-        }
-        g_free (parts);
-        if (search_uri)
-        {
-            if (strstr (search_uri, "%s"))
-                search = g_strdup_printf (search_uri, parts[1]);
-            else
-                search = g_strdup_printf ("%s%s", search_uri, parts[1]);
-        }
-        return search;
+        if (!(parts[1][1] == '\0' && !g_ascii_isalpha (parts[1][0])))
+            if (!strchr (parts[0], ' ') && !strchr (parts[1], ' '))
+                if ((search = g_strconcat ("http://", uri, NULL)))
+                {
+                    g_strfreev (parts);
+                    return sokoke_idn_to_punycode (search);
+                }
     }
-    return sokoke_idn_to_punycode (g_strdup (uri));
+    g_strfreev (parts);
+    /* We don't want to search? So return early. */
+    if (!search_engines)
+        return g_strdup (uri);
+    search = NULL;
+    search_uri = NULL;
+    /* Do we have a keyword and a string? */
+    parts = g_strsplit (uri, " ", 2);
+    if (parts[0] && parts[1])
+        if ((item = katze_array_find_token (search_engines, parts[0])))
+        {
+            gchar* uri_ = g_uri_escape_string (parts[1], " :/", TRUE);
+            search_uri = katze_item_get_uri (item);
+            if (strstr (search_uri, "%s"))
+                search = g_strdup_printf (search_uri, uri_);
+            else
+                search = g_strconcat (search_uri, uri_, NULL);
+            g_free (uri_);
+        }
+    g_strfreev (parts);
+    return search;
 }
 
 void
@@ -763,6 +771,7 @@ sokoke_register_stock_items (void)
     static FatStockItem items[] =
     {
         { STOCK_EXTENSION, NULL, 0, 0, GTK_STOCK_CONVERT },
+        { STOCK_IMAGE, NULL, 0, 0, GTK_STOCK_ORIENTATION_PORTRAIT },
         { STOCK_NEWS_FEED, NULL, 0, 0, GTK_STOCK_INDEX },
         { STOCK_SCRIPT, NULL, 0, 0, GTK_STOCK_EXECUTE },
         { STOCK_STYLE, NULL, 0, 0, GTK_STOCK_SELECT_COLOR },
@@ -779,6 +788,7 @@ sokoke_register_stock_items (void)
         { STOCK_STYLES,         N_("User_styles"), 0, 0, GTK_STOCK_SELECT_COLOR },
         { STOCK_TAB_NEW,        N_("New _Tab"), 0, 0, GTK_STOCK_ADD },
         { STOCK_TRANSFERS,      N_("_Transfers"), 0, 0, GTK_STOCK_SAVE },
+        { STOCK_PLUGINS,        N_("P_lugins"), 0, 0, GTK_STOCK_CONVERT },
         { STOCK_USER_TRASH,     N_("_Closed Tabs and Windows"), 0, 0, "gtk-undo-ltr" },
         { STOCK_WINDOW_NEW,     N_("New _Window"), 0, 0, GTK_STOCK_ADD },
     };
@@ -803,3 +813,70 @@ sokoke_register_stock_items (void)
     gtk_icon_factory_add_default (factory);
     g_object_unref (factory);
 }
+
+/**
+ * sokoke_set_config_dir:
+ * @new_config_dir: an absolute path, or %NULL
+ *
+ * Retrieves and/ or sets the base configuration folder.
+ *
+ * Return value: the configuration folder, or %NULL
+ **/
+const gchar*
+sokoke_set_config_dir (const gchar* new_config_dir)
+{
+    static gchar* config_dir = NULL;
+
+    if (config_dir)
+        return config_dir;
+
+    if (!new_config_dir)
+        config_dir = g_build_filename (g_get_user_config_dir (),
+                                       PACKAGE_NAME, NULL);
+    else
+    {
+        g_return_val_if_fail (g_path_is_absolute (new_config_dir), NULL);
+        katze_assign (config_dir, g_strdup (new_config_dir));
+    }
+
+    return config_dir;
+}
+
+/**
+ * sokoke_remove_path:
+ * @path: an absolute path
+ * @ignore_errors: keep removing even if an error occurred
+ *
+ * Removes the file at @path or the folder including any
+ * child folders and files if @path is a folder.
+ *
+ * If @ignore_errors is %TRUE and @path is a folder with
+ * children, one of which can't be removed, remaining
+ * children will be deleted nevertheless
+ * If @ignore_errors is %FALSE and @path is a folder, the
+ * removal process will cancel immediately.
+ *
+ * Return value: %TRUE on success, %FALSE if an error occurred
+ **/
+gboolean
+sokoke_remove_path (const gchar* path,
+                    gboolean     ignore_errors)
+{
+    GDir* dir = g_dir_open (path, 0, NULL);
+    const gchar* name;
+
+    if (!dir)
+        return g_remove (path) == 0;
+
+    while ((name = g_dir_read_name (dir)))
+    {
+        gchar* sub_path = g_build_filename (path, name, NULL);
+        if (!sokoke_remove_path (sub_path, ignore_errors) && !ignore_errors)
+            return FALSE;
+        g_free (sub_path);
+    }
+
+    g_dir_close (dir);
+    g_rmdir (path);
+    return TRUE;
+}
