@@ -18,7 +18,7 @@
 
 #include <JavaScriptCore/JavaScript.h>
 
-JSValueRef
+gchar*
 sokoke_js_script_eval (JSContextRef js_context,
                        const gchar* script,
                        gchar**      exception);
@@ -127,4 +127,11 @@ sokoke_time_t_to_julian              (const time_t*  timestamp);
 void
 sokoke_register_stock_items          (void);
 
+const gchar*
+sokoke_set_config_dir                (const gchar* new_config_dir);
+
+gboolean
+sokoke_remove_path                   (const gchar* path,
+                                      gboolean     ignore_errors);
+
 #endif /* !__SOKOKE_H__ */
