@@ -64,6 +64,9 @@ enum
     PROP_APP
 };
 
+static void
+midori_history_finalize     (GObject*      object);
+
 static void
 midori_history_set_property (GObject*      object,
                              guint         prop_id,
@@ -83,6 +86,7 @@ midori_history_class_init (MidoriHistoryClass* class)
     GParamFlags flags;
 
     gobject_class = G_OBJECT_CLASS (class);
+    gobject_class->finalize = midori_history_finalize;
     gobject_class->set_property = midori_history_set_property;
     gobject_class->get_property = midori_history_get_property;
 
@@ -185,7 +189,6 @@ midori_history_cursor_or_row_changed_cb (GtkTreeView*   treeview,
         is_page = !KATZE_IS_ARRAY (item) && katze_item_get_uri (item);
         gtk_widget_set_sensitive (history->bookmark, is_page);
         gtk_widget_set_sensitive (history->delete, TRUE);
-        gtk_widget_set_sensitive (history->clear, TRUE);
 
         g_object_unref (item);
     }
@@ -193,7 +196,6 @@ midori_history_cursor_or_row_changed_cb (GtkTreeView*   treeview,
     {
         gtk_widget_set_sensitive (history->bookmark, FALSE);
         gtk_widget_set_sensitive (history->delete, FALSE);
-        gtk_widget_set_sensitive (history->clear, FALSE);
     }
 }
 
@@ -441,7 +443,8 @@ midori_history_insert_item (MidoriHistory* history,
     {
         GtkTreeIter* piter;
         gint64 pday;
-        guint i, n;
+        guint i;
+        KatzeItem* child;
 
         g_signal_connect (item, "add-item",
             G_CALLBACK (midori_history_add_item_cb), history);
@@ -459,14 +462,9 @@ midori_history_insert_item (MidoriHistory* history,
             g_object_unref (item);
             piter = &iter;
         }
-        n = katze_array_get_length (KATZE_ARRAY (item));
-        for (i = 0; i < n; i++)
-        {
-            KatzeItem* child;
-
-            child = katze_array_get_nth_item (KATZE_ARRAY (item), i);
+        i = 0;
+        while ((child = katze_array_get_nth_item (KATZE_ARRAY (item), i++)))
             midori_history_insert_item (history, treestore, piter, child, day);
-        }
     }
     else
     {
@@ -499,7 +497,6 @@ midori_history_set_app (MidoriHistory* history,
         time_t now = time (NULL);
         gint64 day = sokoke_time_t_to_julian (&now);
 
-        /* FIXME: Dereference the app on finalization */
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (history->treeview));
         midori_history_insert_item (history, GTK_TREE_STORE (model),
             NULL, KATZE_ITEM (g_object_ref (history->array)), day);
@@ -725,6 +722,7 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
                 settings = katze_object_get_object (browser, "settings");
                 if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
                     midori_browser_set_current_page (MIDORI_BROWSER (browser), n);
+                g_object_unref (settings);
             }
             i++;
         }
@@ -741,6 +739,7 @@ midori_history_open_in_tab_activate_cb (GtkWidget*     menuitem,
             settings = katze_object_get_object (browser, "settings");
             if (!katze_object_get_boolean (settings, "open-tabs-in-the-background"))
                 midori_browser_set_current_page (MIDORI_BROWSER (browser), n);
+            g_object_unref (settings);
         }
     }
 }
@@ -961,6 +960,16 @@ midori_history_init (MidoriHistory* history)
     history->treeview = treeview;
 }
 
+static void
+midori_history_finalize (GObject* object)
+{
+    MidoriHistory* history = MIDORI_HISTORY (object);
+
+    if (history->app)
+        g_object_unref (history->app);
+    g_object_unref (history->array);
+}
+
 /**
  * midori_history_new:
  *
